@@ -34,6 +34,10 @@ class InvalidReplyError(BadReplyError, ABC):
     could not be parsed into a response."""
 
 
+class UnexpectedIDError (InvalidReplyError, ABC):
+    """A reply received contained an invalid unique identifier."""
+
+
 class MethodNotFoundError(RPCError, ABC):
     """The desired method was not found."""
 
@@ -44,3 +48,6 @@ class InvalidParamsError(RPCError, ABC):
 
 class ServerError(RPCError, ABC):
     """An internal error in the RPC system occurred."""
+
+class TimeoutError(Exception):
+    """No reply received within the timeout period."""
