@@ -12,12 +12,14 @@ Tinyrpc will detect the presence of jsonext and use it automatically.
 
 import json
 import sys
-from typing import Dict, Any, Union, Optional, List, Tuple, Callable
+from tinyrpc.exc import UnexpectedIDError
+from typing import Dict, Any, Union, Optional, List, Tuple, Callable, Generator
 
+from . import default_id_generator
 from .. import (
     RPCBatchProtocol, RPCRequest, RPCResponse, RPCErrorResponse,
     InvalidRequestError, MethodNotFoundError, InvalidReplyError, RPCError,
-    RPCBatchRequest, RPCBatchResponse, InvalidParamsError
+    RPCBatchRequest, RPCBatchResponse, InvalidParamsError,
 )
 
 if 'jsonext' in sys.modules:
@@ -505,13 +507,18 @@ class JSONRPCProtocol(RPCBatchProtocol):
     _ALLOWED_REPLY_KEYS = sorted(['id', 'jsonrpc', 'error', 'result'])
     _ALLOWED_REQUEST_KEYS = sorted(['id', 'jsonrpc', 'method', 'params'])
 
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(
+            self,
+            id_generator: Optional[Generator[object, None, None]] = None,
+            *args,
+            **kwargs
+    ) -> None:
         super(JSONRPCProtocol, self).__init__(*args, **kwargs)
-        self._id_counter = 0
+        self._id_generator = id_generator or default_id_generator()
+        self._pending_replies = []
 
-    def _get_unique_id(self) -> int:
-        self._id_counter += 1
-        return self._id_counter
+    def _get_unique_id(self) -> object:
+        return next(self._id_generator)
 
     def request_factory(self) -> 'JSONRPCRequest':
         """Factory for request objects.
@@ -568,6 +575,7 @@ class JSONRPCProtocol(RPCBatchProtocol):
 
         if not one_way:
             request.unique_id = self._get_unique_id()
+            self._pending_replies.append(request.unique_id)
 
         request.method = method
         if args is not None:
@@ -579,7 +587,7 @@ class JSONRPCProtocol(RPCBatchProtocol):
 
     def parse_reply(
             self, data: bytes
-    ) -> Union['JSONRPCSuccessResponse', 'JSONRPCErrorResponse']:
+    ) -> Union['JSONRPCSuccessResponse', 'JSONRPCErrorResponse', 'JSONRPCBatchResponse']:
         """De-serializes and validates a response.
 
         Called by the client to reconstruct the serialized :py:class:`JSONRPCResponse`.
@@ -599,6 +607,24 @@ class JSONRPCProtocol(RPCBatchProtocol):
         except Exception as e:
             raise InvalidReplyError(e)
 
+        if isinstance(rep, list):
+            # batch request
+            replies = JSONRPCBatchResponse()
+            for subrep in rep:
+                try:
+                    replies.append(self._parse_subreply(subrep))
+                except RPCError as e:
+                    replies.append(e)
+                except Exception as e:
+                    replies.append(InvalidReplyError(e))
+
+            if not replies:
+                raise InvalidReplyError("Empty batch response received.")
+            return replies
+        else:
+            return self._parse_subreply(rep)
+
+    def _parse_subreply(self, rep):
         for k in rep.keys():
             if k not in self._ALLOWED_REPLY_KEYS:
                 raise InvalidReplyError('Key not allowed: %s' % k)
@@ -629,6 +655,12 @@ class JSONRPCProtocol(RPCBatchProtocol):
             response.result = rep.get('result', None)
 
         response.unique_id = rep['id']
+        if response.unique_id not in self._pending_replies:
+            raise UnexpectedIDError(
+                'Reply id does not correspond to any sent requests.'
+            )
+        else:
+            self._pending_replies.remove(response.unique_id)
 
         return response
 
